import es

info = es.AddonInfo()
info.name = "Python event call modifier"
info.basename = "modevents"
info.version = "beta"
info.description = "Allows modifying event variables for Python addon event calls"
info.url = "http://addons.eventscripts.com/addons/view/modevents"

hookers = {}

es.ServerVar('modevents_py', info.version, '%s version'%info.name).makepublic()

class FakeEventVariables(dict):
    def __getitem__(self, key):
        if key in self:
            return str(super(FakeEventVariables, self).__getitem__(key))
        else:
            return es.getEventInfo(key)

class FakeEventCallbacks(dict):
    def __init__(self, eventname, contents):
        super(FakeEventCallbacks, self).__init__(contents)
        self.eventname = eventname
    def __setitem__(self, key, value):
        es.addons.EventListeners.hack_getitem(self.eventname)[key] = value
    def __delitem__(self, key):
        del es.addons.EventListeners.hack_getitem(self.eventname)[key]
    def __iter__(self):
        theiter = super(FakeEventCallbacks, self).__iter__()
        es.event_var.clear()
        if self.eventname in hookers:
            for hacker in hookers[self.eventname]:
                if hacker(es.event_var) == False:
                    raise StopIteration
        return theiter

class FakeEventListeners(dict):
    def has_key(self, eventname):
        return super(FakeEventListeners, self).has_key(eventname)
    def __getitem__(self, eventname):
        return FakeEventCallbacks(eventname,
            super(FakeEventListeners, self).__getitem__(eventname))
    def hack_getitem(self, eventname):
        return super(FakeEventListeners, self).__getitem__(eventname)
        

es.event_var = FakeEventVariables()
es.addons.EventListeners = FakeEventListeners(es.addons.EventListeners)

def add_modder(eventname, callback):
    es.dbgmsg(1, 'modevents.add_modder(%s, %s)'%(eventname, repr(callback)))
    if eventname not in hookers:
        hookers[eventname] = set()
    hookers[eventname].add(callback)

def del_modder(eventname, callback):
    es.dbgmsg(1, 'modevents.del_modder(%s, %s)'%(eventname, repr(callback)))
    hookers[eventname].remove(callback)
